Class: Patricia
- Inherits:
-
Object
- Object
- Patricia
- Defined in:
- ext/rpatricia/rpatricia.c
Defined Under Namespace
Classes: Node
Instance Method Summary collapse
-
#add ⇒ Object
add string.
- #add_node ⇒ Object
- #clear ⇒ Object
-
#destroy ⇒ Object
destroy tree.
- #family ⇒ Object
-
#include? ⇒ Boolean
check existence.
- #initialize_copy(orig) ⇒ Object
-
#match_best ⇒ Object
match prefix.
-
#match_exact ⇒ Object
exact match.
- #nodes ⇒ Object
-
#num_nodes ⇒ Object
derivatives of climb.
-
#remove ⇒ Object
removal.
- #remove_node ⇒ Object
- #search_best ⇒ Object
- #search_exact ⇒ Object
- #show_nodes ⇒ Object
Instance Method Details
#add ⇒ Object
add string
64 65 66 67 68 69 70 71 72 73 74 75 76 77 78 79 80 81 82 83 84 85 86 87 88 89 90 91 92 |
# File 'ext/rpatricia/rpatricia.c', line 64 static VALUE p_add (int argc, VALUE *argv, VALUE self) { VALUE user_data; patricia_tree_t *tree; patricia_node_t *node; prefix_t prefix; if (argc > 2 || argc < 1) return Qnil; Data_Get_Struct(self, patricia_tree_t, tree); my_ascii2prefix(tree, argv[0], &prefix); node = patricia_lookup(tree, &prefix); if (argc == 2) { user_data = argv[1]; /* for backwards compatibility, we always dup and return new strings */ if (TYPE(user_data) == T_STRING) user_data = rb_str_dup(user_data); } else { user_data = rb_str_new(NULL, 0); } PATRICIA_DATA_SET(node, user_data); /* node will be freed when parent is freed */ return wrap_node(node); } |
#add_node ⇒ Object
64 65 66 67 68 69 70 71 72 73 74 75 76 77 78 79 80 81 82 83 84 85 86 87 88 89 90 91 92 |
# File 'ext/rpatricia/rpatricia.c', line 64 static VALUE p_add (int argc, VALUE *argv, VALUE self) { VALUE user_data; patricia_tree_t *tree; patricia_node_t *node; prefix_t prefix; if (argc > 2 || argc < 1) return Qnil; Data_Get_Struct(self, patricia_tree_t, tree); my_ascii2prefix(tree, argv[0], &prefix); node = patricia_lookup(tree, &prefix); if (argc == 2) { user_data = argv[1]; /* for backwards compatibility, we always dup and return new strings */ if (TYPE(user_data) == T_STRING) user_data = rb_str_dup(user_data); } else { user_data = rb_str_new(NULL, 0); } PATRICIA_DATA_SET(node, user_data); /* node will be freed when parent is freed */ return wrap_node(node); } |
#clear ⇒ Object
20 21 22 23 24 25 26 27 28 29 30 |
# File 'ext/rpatricia/rpatricia.c', line 20 static VALUE p_destroy (VALUE self) { patricia_tree_t *tree; Data_Get_Struct(self, patricia_tree_t, tree); Clear_Patricia(tree, dummy); tree->head = NULL; /* Clear_Patricia() should do this, actually */ return Qtrue; } |
#destroy ⇒ Object
destroy tree
20 21 22 23 24 25 26 27 28 29 30 |
# File 'ext/rpatricia/rpatricia.c', line 20 static VALUE p_destroy (VALUE self) { patricia_tree_t *tree; Data_Get_Struct(self, patricia_tree_t, tree); Clear_Patricia(tree, dummy); tree->head = NULL; /* Clear_Patricia() should do this, actually */ return Qtrue; } |
#family ⇒ Object
360 361 362 363 364 365 366 367 368 369 370 371 372 373 |
# File 'ext/rpatricia/rpatricia.c', line 360 static VALUE p_family(VALUE self) { patricia_tree_t *tree; Data_Get_Struct(self, patricia_tree_t, tree); switch (tree->maxbits) { case 32: return sym_AF_INET; case 128: return sym_AF_INET6; } assert(0 && "unknown maxbits, corrupt tree"); return Qnil; } |
#include? ⇒ Boolean
check existence
127 128 129 130 131 132 133 134 135 136 137 138 139 |
# File 'ext/rpatricia/rpatricia.c', line 127 static VALUE p_include (VALUE self, VALUE r_key) { patricia_tree_t *tree; patricia_node_t *node; prefix_t prefix; Data_Get_Struct(self, patricia_tree_t, tree); my_ascii2prefix(tree, r_key, &prefix); node = patricia_search_best(tree, &prefix); return node ? Qtrue : Qfalse; } |
#initialize_copy(orig) ⇒ Object
333 334 335 336 337 338 339 340 341 342 343 344 345 346 347 348 349 350 351 352 353 354 355 356 357 358 |
# File 'ext/rpatricia/rpatricia.c', line 333 static VALUE p_init_copy(VALUE self, VALUE orig) { patricia_tree_t *orig_tree; Data_Get_Struct(orig, patricia_tree_t, orig_tree); if (orig_tree->head) { patricia_tree_t *tree; patricia_node_t *orig_node, *node; VALUE user_data; DATA_PTR(self) = tree = New_Patricia(orig_tree->maxbits); PATRICIA_WALK(orig_tree->head, orig_node) { node = patricia_lookup(tree, orig_node->prefix); assert(node->prefix == orig_node->prefix); user_data = (VALUE)(orig_node->data); if (T_STRING == TYPE(user_data)) user_data = rb_str_dup(user_data); PATRICIA_DATA_SET(node, user_data); } PATRICIA_WALK_END; } return self; } |
#match_best ⇒ Object
match prefix
113 114 115 116 117 118 119 120 121 122 123 124 125 |
# File 'ext/rpatricia/rpatricia.c', line 113 static VALUE p_match (VALUE self, VALUE r_key) { patricia_tree_t *tree; patricia_node_t *node; prefix_t prefix; Data_Get_Struct(self, patricia_tree_t, tree); my_ascii2prefix(tree, r_key, &prefix); node = patricia_search_best(tree, &prefix); return node ? wrap_node(node) : Qnil; } |
#match_exact ⇒ Object
exact match
141 142 143 144 145 146 147 148 149 150 151 152 153 |
# File 'ext/rpatricia/rpatricia.c', line 141 static VALUE p_match_exact (VALUE self, VALUE r_key) { patricia_tree_t *tree; patricia_node_t *node; prefix_t prefix; Data_Get_Struct(self, patricia_tree_t, tree); my_ascii2prefix(tree, r_key, &prefix); node = patricia_search_exact(tree, &prefix); return node ? wrap_node(node) : Qnil; } |
#nodes ⇒ Object
205 206 207 208 209 210 211 212 213 214 215 216 217 218 219 220 221 222 223 224 225 226 227 |
# File 'ext/rpatricia/rpatricia.c', line 205 static VALUE p_nodes (VALUE self) { VALUE buf = rb_str_new(0, 0); char *cbuf; patricia_tree_t *tree; patricia_node_t *node; VALUE hash; Data_Get_Struct(self, patricia_tree_t, tree); hash = rb_hash_new(); if (tree->head) { PATRICIA_WALK(tree->head, node) { rb_str_resize(buf, PATRICIA_MAXSTRLEN); cbuf = RSTRING_PTR(buf); prefix_toa2x(node->prefix, cbuf, 1); rb_str_set_len(buf, strlen(cbuf)); rb_hash_aset(hash, buf, (VALUE)node->data); } PATRICIA_WALK_END; } return hash; } |
#num_nodes ⇒ Object
derivatives of climb
155 156 157 158 159 160 161 162 163 164 165 |
# File 'ext/rpatricia/rpatricia.c', line 155 static VALUE p_num_nodes (VALUE self) { int n; patricia_tree_t *tree; Data_Get_Struct(self, patricia_tree_t, tree); n = tree->head ? patricia_walk_inorder(tree->head, dummy) : 0; return INT2NUM(n); } |
#remove ⇒ Object
removal
94 95 96 97 98 99 100 101 102 103 104 105 106 107 108 109 110 111 |
# File 'ext/rpatricia/rpatricia.c', line 94 static VALUE p_remove (VALUE self, VALUE r_key) { patricia_tree_t *tree; patricia_node_t *node; prefix_t prefix; Data_Get_Struct(self, patricia_tree_t, tree); my_ascii2prefix(tree, r_key, &prefix); node = patricia_search_exact(tree, &prefix); if (node) { patricia_remove (tree, node); return Qtrue; } else { return Qfalse; } } |
#remove_node ⇒ Object
94 95 96 97 98 99 100 101 102 103 104 105 106 107 108 109 110 111 |
# File 'ext/rpatricia/rpatricia.c', line 94 static VALUE p_remove (VALUE self, VALUE r_key) { patricia_tree_t *tree; patricia_node_t *node; prefix_t prefix; Data_Get_Struct(self, patricia_tree_t, tree); my_ascii2prefix(tree, r_key, &prefix); node = patricia_search_exact(tree, &prefix); if (node) { patricia_remove (tree, node); return Qtrue; } else { return Qfalse; } } |
#search_best ⇒ Object
113 114 115 116 117 118 119 120 121 122 123 124 125 |
# File 'ext/rpatricia/rpatricia.c', line 113 static VALUE p_match (VALUE self, VALUE r_key) { patricia_tree_t *tree; patricia_node_t *node; prefix_t prefix; Data_Get_Struct(self, patricia_tree_t, tree); my_ascii2prefix(tree, r_key, &prefix); node = patricia_search_best(tree, &prefix); return node ? wrap_node(node) : Qnil; } |
#search_exact ⇒ Object
141 142 143 144 145 146 147 148 149 150 151 152 153 |
# File 'ext/rpatricia/rpatricia.c', line 141 static VALUE p_match_exact (VALUE self, VALUE r_key) { patricia_tree_t *tree; patricia_node_t *node; prefix_t prefix; Data_Get_Struct(self, patricia_tree_t, tree); my_ascii2prefix(tree, r_key, &prefix); node = patricia_search_exact(tree, &prefix); return node ? wrap_node(node) : Qnil; } |
#show_nodes ⇒ Object
177 178 179 180 181 182 183 184 185 186 187 188 189 190 191 192 193 194 195 196 197 198 199 200 201 202 203 |
# File 'ext/rpatricia/rpatricia.c', line 177 static VALUE p_print_nodes (int argc, VALUE *argv, VALUE self) { ID id_printf = rb_intern("printf"); VALUE fmt = rb_str_new2("node: %s\n"); VALUE buf = rb_str_new(0, 0); char *cbuf; patricia_tree_t *tree; patricia_node_t *node; VALUE out; Data_Get_Struct(self, patricia_tree_t, tree); rb_scan_args(argc, argv, "01", &out); if (NIL_P(out)) out = rb_stdout; if (tree->head) { PATRICIA_WALK(tree->head, node) { rb_str_resize(buf, PATRICIA_MAXSTRLEN); cbuf = RSTRING_PTR(buf); prefix_toa2x(node->prefix, cbuf, 1); rb_str_set_len(buf, strlen(cbuf)); rb_funcall(out, id_printf, 2, fmt, buf); } PATRICIA_WALK_END; } return Qtrue; } |